/* serielle Kommunikation fr das STM32F103ZET6-Board */

#include "startup.h"
#include "STM32F103.h"

/* Funktionsprinzip:
Der eigentliche Datenverkehr ber die diversen UART's wird
weitgehend interruptgesteuert erledigt. Als Schnittstelle
zu den Anwendungen dienen Sende- und Empfangspuffer, die mit
entsprechenden Funktionen abgefragt und befllt werden
knnen.
 */

#define TUBUF  struct T_UARTBUF
struct T_UARTBUF
{ char InBuf[16];
  char OutBuf[256];
  int  InWP, InRP;
  int  OutWP, OutRP;
};

/* hier nur die wirklich bentigten Puffer entkommentieren */
TUBUF U1Buf;
TUBUF U2Buf;
TUBUF U3Buf;
/* TUBUF U4Buf; */
/* TUBUF U5Buf; */


TUBUF* GetUsartBuffer (word usartnum)
{ switch(usartnum)
  { case 1: return &U1Buf;
    case 2: return &U2Buf;
    case 3: return &U3Buf;
/*    case 4: return UART4; */
/*    case 5: return UART5; */
  }
  return 0;
}


struct T_USART* GetUsart (word usartnum)
{ switch(usartnum)
  { case 1: return USART1;
    case 2: return USART2;
    case 3: return USART3;
    case 4: return UART4;
    case 5: return UART5;
  }
  return 0;
}


/************** NVIC Pos. 37 = USART1 Interrupt-Handler ****************/
__irq void Usart1_IntHandler (void)
{ volatile unsigned long L;
  char c;
  int  i;

  L = USART1->SR;
  if (L & (1<<5))    /* RXNE = Receiver not empty */
  { c = USART1->DR;  /* read date, thereby clear RXNE */
    i = U1Buf.InWP;
    U1Buf.InBuf[i] = c;
    ++i;
    if (i >= sizeof(U1Buf.InBuf)) i = 0;
  }

  if (L & (1<<7))            /* TXE = Senderegister ist leer geworden */
  { USART1->SR &= ~(1<<7);   /* TXE lschen, geht denn das? */
    if (U1Buf.OutWP!=U1Buf.OutRP)
    { i = U1Buf.OutRP;             /* nchstes Zeichen senden */
      c = U1Buf.OutBuf[i];
      ++i;
      if (i >= sizeof(U1Buf.OutBuf)) i = 0;
      U1Buf.OutRP = i;
      USART1->DR = c;
    }
    else                     /* wenn nix zu senden, dann Interrupts ausschalten */
    { USART1->CR1 &= ~((1<<7)|(1<<6)); /* TXIE und TCIE lschen */
    }
  }
}



/*  USART aufsetzen
 Baudrate errechnen.
 Die Formel ist:  Baudrate = Busclock / BRR;
 USART1 ist an APB2 (max. 48 / 72 MHz),
 alle anderen USART's sind an APB1 (max. 24 / 36 MHz)
 Bei den hier verwendeten 36 MHz als APB-Clock
 ergibt das fr 9600 Bd --> BRR = 3750;
 */
void InitSerial (word usartnum, long BRR_Value)
{ struct T_USART* P;
  TUBUF* T;
  P = GetUsart(usartnum);
  T = GetUsartBuffer(usartnum);
  if ((P)&&(T))
  { P->BRR = BRR_Value;
    P->CR1 =   (1<<13)|     /* Usart enable */
               (1<<3)|      /* TX enable */
               (1<<2)|      /* RX enable */
0;
/*	       (1<<5);*/      /* RX Interrupt enable */
    P->CR2 = 0;
    P->CR3 = 0;
    P->GTPR = 513;
    T->InWP =
    T->InRP =
    T->OutWP =
    T->OutRP = 0;

  }
}


byte RxAvail (word usartnum)
{ struct T_USART* P;
  P = GetUsart(usartnum);
  if (P)
  { if (P->SR & (1<<5)) return 1;}
  return 0;
}

char GetChar (word usartnum)
{ struct T_USART* P;
  char c;
  c = 0;
  if (RxAvail(usartnum))
  { P = GetUsart(usartnum);
    if (P) c = P->DR;
  }
  return c;
}

char Char_Out (char c, word usartnum)
{ struct T_USART* P;
  long L;
  P = GetUsart(usartnum);
  if (P)
  { L = 10000000;
    while (--L)
    { if (P->SR & (1<<7)) goto _raushier;
    }
    _raushier:
    P->DR = c;
  }
  return c;
}

void String_Out (char* P, char usartnum)
{ while (*P) Char_Out(*P++, usartnum); }

void CRLF_Out(char usartnum)
{ Char_Out(13, usartnum);
  Char_Out(10, usartnum);
}


