/*  Portbelegungen und Konfiguration:
    =================================

Die Kombination der einzelnen #define...
zu mehreren const... erfolgt in "config.c"

 */


#ifndef CONFIG_INCLUDED
#define CONFIG_INCLUDED

#include "startup.h"
#include "STM32F103.h"

/******************************************************
zu jedem Pin gibt es 2 Bit CNF und 2 Bit MODE:
je nach MODE hat CNF unterschiedliche Bedeutung:
MODE: 0=input,
      1=output(10MHz),
      2=output(2MHz),
      3=output(50MHz)
a) wenn MODE = 0 (input) dann:
   CNF: 0=analog,
        1=input,
	2=input mit pullup/dn,
	3=verboten
   Pullup / Pulldown wird per OutputDataRegister eingestellt,
   also quasi ein Vorwiderstand zwischen Pin und ODR

b) wenn MODE = 1 oder 2 oder 3, dann:
   CNF: 0=GPIO,
        1=GPIO(open drain),
	2=alternate fkt.,
	3=alternate(open drain)
*/

/* Merke: Die folgenden Hilfsbezeichnungen sind in STM32F103.h deklariert.  */

/* Port A: */
#define wf_PA0   (in_gpi)             /*  in   wakeup von Seiko-Epson-RTC oder extern */
#define wf_PA1   (in_pupd)            /*  ?    an Testpunkt */
#define wf_PA2   (out2_altfu)         /* TXD2 normaler Kommunikationsport */
#define wf_PA3   (in_gpi)             /* RXD2 9600 Bd 8N1 an DSUB9, */
#define wf_PA4   (out2_altfu)         /* ?    (DAC) */
#define wf_PA5   (out2_altfu)         /* ?    (DAC) */
#define wf_PA6   (in_pupd)            /* ? */
#define wf_PA7   (in_pupd)            /* ?    an Messerleiste */
#define wf_PA8   (out50_altfu)        /* out  MasterClock Out */
#define wf_PA9   (out2_altfu)         /* TXD1 */
#define wf_PA10  (in_gpi)             /* RXD1 an Programmierport */
#define wf_PA11  (out50_altfu)        /* USB D- */
#define wf_PA12  (out50_altfu)        /* USB D+ */
#define wf_PA13  (out50_altfu)        /* JTAG TMS */
#define wf_PA14  (out50_altfu)        /* JTAG TCK */
#define wf_PA15  (out50_altfu)        /* JTAG TDI */


/* Port B: */
#define wf_PB0   (in_pupd)            /* ?    an Testpunkt */
#define wf_PB1   (in_pupd)            /* ?    an Testpunkt */
#define wf_PB2   (out50_altfu)        /* out  Enable fr USB-rstout */
#define wf_PB3   (out50_altfu)        /* JTAG TDO */
#define wf_PB4   (out50_altfu)        /* JTAG RST */
#define wf_PB5   (out50_altfu)        /* interrupt CLK vom PS/2 Mini-Din */
#define wf_PB6   (out2_altfu_od)      /* I2C1 SCL */
#define wf_PB7   (out2_altfu_od)      /* I2C1 SDA */
#define wf_PB8   (in_gpi)             /* in   DATA vom PS/2 Mini-Din */
#define wf_PB9   (in_pupd)            /* ?    an Testpunkt */
#define wf_PB10  (out2_altfu)         /* TXD3 */
#define wf_PB11  (out2_altfu)         /* RXD3 an Messerleiste TTL-Pegel */
#define wf_PB12  (input+conf2)        /* SPI2=I2S  Audio: L/R WCLK */
#define wf_PB13  (input+conf2)        /* SPI2=I2S  SCK */
#define wf_PB14  (input+conf2)        /* SPI2=I2S  MISO */
#define wf_PB15  (input+conf2)        /* SPI2=I2S  MOSI */


/* Port C: */
#define wf_PC0   (input+conf0)        /* adc  an Messerleiste */
#define wf_PC1   (input+conf0)        /* adc  an Messerleiste */
#define wf_PC2   (input+conf0)        /* adc  an Messerleiste */
#define wf_PC3   (input+conf0)        /* adc  an Messerleiste */
#define wf_PC4   (input+conf0)        /* adc  an Messerleiste */
#define wf_PC5   (input+conf0)        /* adc  an Messerleiste */
#define wf_PC6   (out50+conf2)        /* SPI2=I2S  MCLK */
#define wf_PC7   (input+conf2)        /* ?    an Testpunkt */
#define wf_PC8   (in_gpi) /* (out50_altfu)*/        /* SDCard D0 */
#define wf_PC9   (out50_altfu)        /* SDCard D1 */
#define wf_PC10  (out50_altfu)        /* SDCard D2 */
#define wf_PC11 (out50_gpo) /*  (out50_altfu) */       /* SDCard D3 */
#define wf_PC12 (out50_gpo) /*  (out50_altfu) */       /* SDCard CLK */
#define wf_PC13  (out2_altfu)         /* an RTC-Bein     Pin 7, nach Reset=GPIO */
#define wf_PC14  (out2_altfu)         /* an OSC32kHz IN  Pin 8  nach Reset=GPIO */
#define wf_PC15  (out2_altfu)         /* an OSC32kHz OUT Pin 9  nach Reset=GPIO */


/* Port D: */
#define wf_PD0   (out50+conf2)        /* Bus    D2 */
#define wf_PD1   (out50+conf2)        /* Bus    D3 */
#define wf_PD2   (out50_gpo) /*  (out50_altfu) */        /* SDCard CMD */
#define wf_PD3   (out50+conf2)        /* Bus    Clk */
#define wf_PD4   (out50+conf2)        /* Bus    OE */
#define wf_PD5   (out50+conf2)        /* Bus    WE */
#define wf_PD6   (out50+conf2)        /* Bus    Wait */
#define wf_PD7   (out50+conf2)        /* Bus    NE1 */
#define wf_PD8   (out50+conf2)        /* Bus    D13 */
#define wf_PD9   (out50+conf2)        /* Bus    D14 */
#define wf_PD10  (out50+conf2)        /* Bus    D15 */
#define wf_PD11  (out50+conf2)        /* Bus    A16 */
#define wf_PD12  (out50+conf2)        /* Bus    A17 */
#define wf_PD13  (out50+conf2)        /* Bus    A18 */
#define wf_PD14  (out50+conf2)        /* Bus    D0 */
#define wf_PD15  (out50+conf2)        /* Bus    D1 */


/* Port E: */
#define wf_PE0   (out50+conf2)        /* Bus    LB */
#define wf_PE1   (out50+conf2)        /* Bus    UB */
#define wf_PE2   (out50+conf0)        /* Trace  CK */
#define wf_PE3   (out50+conf0)        /* Trace  D0 */
#define wf_PE4   (out50+conf0)        /* Trace  D1 */
#define wf_PE5   (out50+conf0)        /* Trace  D2 */
#define wf_PE6   (out50+conf0)        /* Trace  D3 */
#define wf_PE7   (out50+conf2)        /* Bus    D4 */
#define wf_PE8   (out50+conf2)        /* Bus    D5 */
#define wf_PE9   (out50+conf2)        /* Bus    D6 */
#define wf_PE10  (out50+conf2)        /* Bus    D7 */
#define wf_PE11  (out50+conf2)        /* Bus    D8 */
#define wf_PE12  (out50+conf2)        /* Bus    D9 */
#define wf_PE13  (out50+conf2)        /* Bus    D10 */
#define wf_PE14  (out50+conf2)        /* Bus    D11 */
#define wf_PE15  (out50+conf2)        /* Bus    D12 */


/* Port F: */
#define wf_PF0   (out50+conf2)        /* Bus    A0 */
#define wf_PF1   (out50+conf2)        /* Bus    A1 */
#define wf_PF2   (out50+conf2)        /* Bus    A2 */
#define wf_PF3   (out50+conf2)        /* Bus    A3 */
#define wf_PF4   (out50+conf2)        /* Bus    A4 */
#define wf_PF5   (out50+conf2)        /* Bus    A5 */
#define wf_PF6   (input+conf2)        /* ?      an Micromatch */
#define wf_PF7   (input+conf2)        /* ?      an Micromatch */
#define wf_PF8   (input+conf2)        /* ?      an Micromatch */
#define wf_PF9   (input+conf2)        /* ?      an Micromatch */
#define wf_PF10  (input+conf2)        /* ?      an Micromatch */
#define wf_PF11  (input+conf2)        /* ?      an Testpunkt */
#define wf_PF12  (out50+conf2)        /* Bus    A6 */
#define wf_PF13  (out50+conf2)        /* Bus    A7 */
#define wf_PF14  (out50+conf2)        /* Bus    A8 */
#define wf_PF15  (out50+conf2)        /* Bus    A9 */


/* Port G: */
#define wf_PG0   (out50+conf2)        /* Bus    A10 */
#define wf_PG1   (out50+conf2)        /* Bus    A11 */
#define wf_PG2   (out50+conf2)        /* Bus    A12 */
#define wf_PG3   (out50+conf2)        /* Bus    A13 */
#define wf_PG4   (out50+conf2)        /* Bus    A14 */
#define wf_PG5   (out50+conf2)        /* Bus    A15 */
#define wf_PG6   (out50_gpo)          /* out    an Testpunkt */
#define wf_PG7   (input+conf2)        /* in (int?) fr SDCard insert detect */
#define wf_PG8   (input+conf2)        /* in     fr SDCard write protect */
#define wf_PG9   (out50+conf2)        /* Bus    NE2 */
#define wf_PG10  (input+conf2)        /* ?      an Testpunkt */
#define wf_PG11  (input+conf2)        /* ?      an Testpunkt */
#define wf_PG12  (input+conf2)        /* ?      an Testpunkt */
#define wf_PG13  (input+conf2)        /* ?      an Testpunkt */
#define wf_PG14  (input+conf2)        /* ?      an Testpunkt */
#define wf_PG15  (input+conf2)        /* in     USB (conn) detect VUSB */


/* Remap-setup */
#define SPI1_REMAP          0     /* PA4,5,6,7 --> PA15,3,4,5 */
#define I2C1_REMAP          0     /* PB6,7 --> PB8,9 */
#define USART1_REMAP        0     /* PA9,10 --> PB6,7 */
#define USART2_REMAP        0     /* PA0,1,2,3,4 --> PD3,4,5,6,7 */
#define USART3_REMAP0       0     /* PB10,11,12 --> PC10,11,12 */
#define USART3_REMAP1       0     /* PB10,11,12,13,14,15 --> PD8,9,10,11,12 */
#define TIM1_REMAP0         0     /*  */
#define TIM1_REMAP1         0     /*  */
#define TIM2_REMAP0         0     /*  */
#define TIM2_REMAP1         0     /*  */
#define TIM3_REMAP0         0     /*  */
#define TIM3_REMAP1         0     /*  */
#define TIM4_REMAP          0     /*  */
#define CAN1_REMAP0         0     /*  */
#define CAN1_REMAP1         0     /*  */
#define PD01_REMAP          0     /*  */
#define TIM5CH4_REMAP       0     /*  */
#define ADC1_ETRGINJ_REMAP  0     /*  */
#define ADC1_ETRGREG_REMAP  0     /*  */
#define ADC2_ETRGINJ_REMAP  0     /*  */
#define ADC3_ETRGREG_REMAP  0     /*  */
#define SWJ_CFG0            0     /*  */
#define SWJ_CFG1            0     /*  */
#define SWJ_CFG2            0     /*  */


#define wf_AFIO_MAPR  (SPI1_REMAP)|(I2C1_REMAP<<1)|(USART1_REMAP<2)|(USART2_REMAP<<3)|(USART3_REMAP0<<4)|(USART3_REMAP1<<5)|(TIM1_REMAP0<<6)| \
                      (TIM1_REMAP1<<7)|(TIM2_REMAP0<8)|(TIM2_REMAP1<9)|(TIM3_REMAP0<10)|(TIM3_REMAP1<<11)|(TIM4_REMAP<<12)|(CAN1_REMAP0<<13)| \
                      (CAN1_REMAP1<<14)|(PD01_REMAP<<15)|(TIM5CH4_REMAP<<16)|(ADC1_ETRGINJ_REMAP<<17)|(ADC1_ETRGREG_REMAP<<18)|(ADC2_ETRGINJ_REMAP<<19)| \
                      (ADC3_ETRGREG_REMAP<<20)| (SWJ_CFG0<<24)|(SWJ_CFG1<<25)|(SWJ_CFG2<<26)


#define wf_GPIOA_CRL  (wf_PA0 | (wf_PA1<<4) | (wf_PA2<<8)  | (wf_PA3<<12)  | (wf_PA4<<16)  | (wf_PA5<<20)  | (wf_PA6<<24)  | (((unsigned)wf_PA7)<<28))
#define wf_GPIOA_CRH  (wf_PA8 | (wf_PA9<<4) | (wf_PA10<<8) | (wf_PA11<<12) | (wf_PA12<<16) | (wf_PA13<<20) | (wf_PA14<<24) | (((unsigned)wf_PA15)<<28))

#define wf_GPIOB_CRL  (wf_PB0 | (wf_PB1<<4) | (wf_PB2<<8)  | (wf_PB3<<12)  | (wf_PB4<<16)  | (wf_PB5<<20)  | (wf_PB6<<24)  | (((unsigned)wf_PB7)<<28))
#define wf_GPIOB_CRH  (wf_PB8 | (wf_PB9<<4) | (wf_PB10<<8) | (wf_PB11<<12) | (wf_PB12<<16) | (wf_PB13<<20) | (wf_PB14<<24) | (((unsigned)wf_PB15)<<28))

#define wf_GPIOC_CRL  (wf_PC0 | (wf_PC1<<4) | (wf_PC2<<8)  | (wf_PC3<<12)  | (wf_PC4<<16)  | (wf_PC5<<20)  | (wf_PC6<<24)  | (((unsigned)wf_PC7)<<28))
#define wf_GPIOC_CRH  (wf_PC8 | (wf_PC9<<4) | (wf_PC10<<8) | (wf_PC11<<12) | (wf_PC12<<16) | (wf_PC13<<20) | (wf_PC14<<24) | (((unsigned)wf_PC15)<<28))

#define wf_GPIOD_CRL  (wf_PD0 | (wf_PD1<<4) | (wf_PD2<<8)  | (wf_PD3<<12)  | (wf_PD4<<16)  | (wf_PD5<<20)  | (wf_PD6<<24)  | (((unsigned)wf_PD7)<<28))
#define wf_GPIOD_CRH  (wf_PD8 | (wf_PD9<<4) | (wf_PD10<<8) | (wf_PD11<<12) | (wf_PD12<<16) | (wf_PD13<<20) | (wf_PD14<<24) | (((unsigned)wf_PD15)<<28))

#define wf_GPIOE_CRL  (wf_PE0 | (wf_PE1<<4) | (wf_PE2<<8)  | (wf_PE3<<12)  | (wf_PE4<<16)  | (wf_PE5<<20)  | (wf_PE6<<24)  | (((unsigned)wf_PE7)<<28))
#define wf_GPIOE_CRH  (wf_PE8 | (wf_PE9<<4) | (wf_PE10<<8) | (wf_PE11<<12) | (wf_PE12<<16) | (wf_PE13<<20) | (wf_PE14<<24) | (((unsigned)wf_PE15)<<28))

#define wf_GPIOF_CRL  (wf_PF0 | (wf_PF1<<4) | (wf_PF2<<8)  | (wf_PF3<<12)  | (wf_PF4<<16)  | (wf_PF5<<20)  | (wf_PF6<<24)  | (((unsigned)wf_PF7)<<28))
#define wf_GPIOF_CRH  (wf_PF8 | (wf_PF9<<4) | (wf_PF10<<8) | (wf_PF11<<12) | (wf_PF12<<16) | (wf_PF13<<20) | (wf_PF14<<24) | (((unsigned)wf_PF15)<<28))

#define wf_GPIOG_CRL  (wf_PG0 | (wf_PG1<<4) | (wf_PG2<<8)  | (wf_PG3<<12)  | (wf_PG4<<16)  | (wf_PG5<<20)  | (wf_PG6<<24)  | (((unsigned)wf_PG7)<<28))
#define wf_GPIOG_CRH  (wf_PG8 | (wf_PG9<<4) | (wf_PG10<<8) | (wf_PG11<<12) | (wf_PG12<<16) | (wf_PG13<<20) | (wf_PG14<<24) | (((unsigned)wf_PG15)<<28))


/* Power- bzw. Takt-Konfiguration */


/***** AHB **************/
#define en_DMA1      1
#define en_DMA2      2
#define en_SRAM      4
#define en_FLASH    (1<<4)
#define en_CRC      (1<<6)
#define en_FSMC     (1<<8)
#define en_SDIO     (1<<10)

#define wf_AHBENR   (en_SDIO | en_FSMC | en_CRC | en_FLASH | en_SRAM | en_DMA2 | en_DMA1)


/***** APB1 ************/
#define en1_DAC     0    /*  */
#define en1_PWR     1    /* Power Interface */
#define en1_BKP     1    /* Backup Interface */
#define en1_CAN     0    /*  */
#define en1_USB     0    /*  */
#define en1_I2C2    0    /*  */
#define en1_I2C1    0    /*  */
#define en1_UART5   0    /*  */
#define en1_UART4   0    /* konkurriert mit SDIO */
#define en1_USART3  1    /*  */
#define en1_USART2  1    /*  */
#define en1_SPI3    0    /*  */
#define en1_SPI2    0    /*  */
#define en1_WWD     0    /* Watchdog */
#define en1_TIM7    0    /*  */
#define en1_TIM6    0    /*  */
#define en1_TIM5    0    /*  */
#define en1_TIM4    0    /*  */
#define en1_TIM3    0    /*  */
#define en1_TIM2    0    /*  */

#define wf_APB1ENR  (en1_TIM2)   | (en1_TIM3<<1)  | (en1_TIM4<<2)  | (en1_TIM5<<3) | (en1_TIM6<<4) | (en1_TIM7<<5) | \
                    (en1_WWD<<11) | (en1_SPI2<<14) | (en1_SPI3<<15) | \
		    (en1_USART2<<17) | (en1_USART3<<18) | (en1_UART4<<19) | (en1_UART5<<20) | \
		    (en1_I2C1<<21) | (en1_I2C2<<22) | \
		    (en1_USB<<23)  | (en1_CAN<<25) | (en1_BKP<<27) | (en1_PWR<<28) | (en1_DAC<<29)


#define en2_ADC3    0    /*  */
#define en2_USART1  1    /*  */
#define en2_TIM8    0    /* konkurriert mit SDIO */
#define en2_SPI1    0    /*  */
#define en2_TIM1    0    /*  */
#define en2_ADC2    0    /*  */
#define en2_ADC1    0    /*  */
#define en2_IOPG    1    /* Port G */
#define en2_IOPF    1    /* Port F */
#define en2_IOPE    1    /* Port E */
#define en2_IOPD    1    /* Port D */
#define en2_IOPC    1    /* Port C */
#define en2_IOPB    1    /* Port B */
#define en2_IOPA    1    /* Port A */
#define en2_AFIO    1    /* Alternative Portbelegungen */

#define wf_APB2ENR  (en2_AFIO) | (en2_IOPA<<2) | (en2_IOPB<<3) | (en2_IOPC<<4) | (en2_IOPD<<5) | (en2_IOPE<<6) | (en2_IOPF<<7) | (en2_IOPG<<8) | \
		    (en2_ADC1<<9) | (en2_ADC2<<10) | (en2_TIM1<<11) | (en2_SPI1<<12) | (en2_TIM8<<13) | (en2_USART1<<14) | (en2_ADC3<<15)

/* Sonstiges:
Der Quarz fr HSE ist 6.000 MHz
Der Quarz fr LSE ist 32.768 kHz

Die CPU luft mit 72 MHz
Die Peripherie luft mit 36 MHz (beide Strnge APB1 und APB2)

*/


extern void Do_Config (void);


#endif
